package cz.zcu.kiv.aswi.svk.lokatory.nmeatimeredlocator;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class Loader extends Thread {

	private List<GeoPoint> locations;
	private int id;

	private DateFormat dateFormatParameters = new SimpleDateFormat("EEE, d. MMM yyyy HH:mm:ss");
	private DateFormat dateFormatNMEALong = new SimpleDateFormat("ddMMyyHHmmss.SSS");
	private DateFormat dateFormatNMEAShort = new SimpleDateFormat("ddMMyyHHmmss");
	private DateFormat timerStartStopFormat = new SimpleDateFormat("HH:mm");

	private BufferedReader input;

	private long dateBegin = 0;
	private long dateEnd = Long.MAX_VALUE;
	private long timerStart = 0;
	private long timerStop = 0;

	private final Map<String, Object> parameters;


	public Loader(List<GeoPoint> locations, Map<String, Object> parameters, int id) {
		super();
		this.locations = locations;
		this.parameters = parameters;
		this.id = id;

		try {
			input = new BufferedReader(new FileReader(parameters.get("file").toString()));
		} catch (Exception e) {
			throw new IllegalArgumentException("Error with opening file.", e);
		}

		// nastaveni locale pro parsovani datumu
		Locale.setDefault(Locale.US);

		//nacteni parametru begin a end
		try {
			dateBegin = dateFormatParameters.parse(parameters.get("begin").toString()).getTime();
		} catch (Exception e) {
			//pouziji se vychozi hodnoty
		}
		try {
			dateEnd = dateFormatParameters.parse(parameters.get("end").toString()).getTime();
		} catch (Exception e) {
			//pouziji se vychozi hodnoty
		}
		if (!parameters.containsKey("start"))
			throw new IllegalArgumentException("Chyb povinn parametr start!");

		try {
			//TODO vraci to o hodinu min, proc?
			timerStart = timerStartStopFormat.parse(parameters.get("start").toString()).getTime() + 3600000;
		} catch (Exception e) {
			throw new IllegalArgumentException("Parametr start nem sprvn formt!");
		}
		if (!parameters.containsKey("stop"))
			throw new IllegalArgumentException("Chyb povinn parametr stop!");

		try {
			//TODO vraci to o hodinu min, proc?
			timerStop = timerStartStopFormat.parse(parameters.get("stop").toString()).getTime() + 3600000;
		} catch (Exception e) {
			throw new IllegalArgumentException("Parametr stop nem sprvn formt!");
		}

	}

	@Override
	public void run() {
		String line;
		String[] words;
		long lastTimeStamp = 0;

		try {
			while(true) {
				while(getTime() < timerStart || getTime() > timerStop) {
					sleep(5000);
				}

				while ((line = input.readLine()) != null) {
					if ("$GPRMC".equals(line.substring(0, 6))) {
						words = line.split(",");
						long timeStamp = parseTimeStamp(words[9] + words[1]);

						if (lastTimeStamp == 0) {
							//System.out.println((getTime() / 3600000));
							//System.out.println((timerStart / 3600000));

							long addTimeStamp = getTime() - timerStart;
							lastTimeStamp = timeStamp + addTimeStamp - 1000;
						}

						if (timeStamp < dateBegin) continue;
						if (timeStamp > dateEnd) break;

						if (timeStamp - lastTimeStamp >= 1000) {
							double latitude = parseLocation(words[3], words[4].charAt(0));
							double longitude = parseLocation(words[5], words[6].charAt(0));

							if (lastTimeStamp == 0)
								lastTimeStamp = timeStamp - 1000;
							saveLocation(timeStamp, latitude, longitude, timeStamp - lastTimeStamp);
							lastTimeStamp = locations.get(id).getTimeStamp();
						}
					}
				}

				locations.set(id, null);
				input.close();

				lastTimeStamp = 0;

				try {
					input = new BufferedReader(new FileReader(parameters.get("file").toString()));
				} catch (Exception e) {
					throw new IllegalArgumentException("Error with opening file.", e);
				}

				if (getTime() < timerStop)
					sleep(timerStop - getTime() + 1000);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
		}

		locations.set(id, null);
	}

	private double parseLocation(String location, char latLonChar) {
		double value = Double.parseDouble(location);
		int degree = (int) (value / 100);
		value-= degree * 100;

		if (Character.toUpperCase(latLonChar) == 'S' || Character.toUpperCase(latLonChar) == 'W')
			degree = -degree;

		return ((double) degree) + value / 60;
	}

	private long parseTimeStamp(String timeStamp) {
		try {
			return dateFormatNMEALong.parse(timeStamp).getTime();
		} catch (ParseException e) {
			try {
				return dateFormatNMEAShort.parse(timeStamp).getTime();
			} catch (ParseException e1) {
				return 0;
			}
		}
	}

	private long getTime() {
		Calendar calendar = Calendar.getInstance();
		//System.out.println(calendar.get(Calendar.HOUR_OF_DAY) + ":" + calendar.get(Calendar.MINUTE) + ":" + calendar.get(Calendar.SECOND) + "." + calendar.get(Calendar.MILLISECOND));

		long miliseconds = calendar.get(Calendar.MILLISECOND);
		miliseconds+= calendar.get(Calendar.SECOND) * 1000;
		miliseconds+= calendar.get(Calendar.MINUTE) * 60000;
		miliseconds+= calendar.get(Calendar.HOUR_OF_DAY) * 3600000;

		return miliseconds;
	}

	synchronized private void saveLocation(long timestamp, double latitude, double longitude, long waitTime) throws InterruptedException {
		locations.set(id, new GeoPoint(timestamp, latitude, longitude));
		this.wait(waitTime);
	}
}
